<?php
/**
Class to connect to a mailbox, also holds the messages of the mailbox as message objects
*/
class hs_imap_mailbox
{
    public $conn = '';

    //IMAP Connection

    public $conn_uri = '';

    public $debug = false;

    //Debug mode

    public $errors = '';

    //IMAP errors

    public $messages = [];

    //Message objects

    public $mailbox_info = [];

    //Array of HS mailbox details

    public $orig_msg_ids = [];

    //Array of ID's for messages originally in mailbox

    public $stuck_emails = [];

    //Array of stuck email ID's (message ID + from)

    public $writable_upload_dir = false;	//flag on if the upload dir is writable

    /**
     * Constructor.
     *
     * @param     string   $uri
     * @return    none
     */
    public function __construct($uri, $debug = false, $mailbox_info = [])
    {
        $this->mailbox_info = $mailbox_info;
        $this->debug = $debug;
        $this->writable_upload_dir = is_writable(ini_get('upload_tmp_dir'));
        $this->connect($uri);
        $this->errors = implode(';', imap_errors());
    }

    /**
     * Establish IMAP connection.
     *
     * @param     string   $uri
     * @return    none
     */
    public function connect($uri)
    {
        if (! class_exists('Net_URL') && ! @include_once('Net/URL.php')) {
            if ($this->debug) {
                echo '>>>> Inclusion of Net_URL not successful.';
            }
            $this->conn = false;

            return false;
        }

        $opt = null;

        $net_url = new Net_URL($uri);

        $uri = '{'.$net_url->host;

        if (! empty($net_url->port)) {
            $uri .= ':'.$net_url->port;
        }

        $secure = ('tls' == substr($net_url->anchor, 0, 3)) ? '' : '/ssl';

        $uri .= ('s' == (substr($net_url->protocol, -1))) ? '/'.substr($net_url->protocol, 0, 4).$secure : '/'.$net_url->protocol;

        if (! empty($net_url->anchor)) {
            $uri .= '/'.$net_url->anchor;
        }

        $uri .= '}';

        // Trim off the leading slash '/'
        if (! empty($net_url->path)) {
            $uri .= substr($net_url->path, 1, (utf8_strlen($net_url->path) - 1));
        }

        $this->conn_uri = $uri;

        if (false === ($this->conn = @imap_open($uri, urldecode($net_url->user), urldecode($net_url->pass), $opt))) {
            $this->conn = false;
            if ($this->debug) {
                echo '>>>> Connection Error: '.implode(';', array_unique(imap_errors()))."\n";
            }

            return false;
        } else {

            //Everything went OK with conn so grab list of stuck emails in mailbox
            $stuckrs = $GLOBALS['DB']->Execute('SELECT * FROM HS_Mailboxes_StuckEmails WHERE HS_Mailboxes_StuckEmails.xMailbox = ?', [$this->mailbox_info['xMailbox']]);

            //Create list of stuck emails and all msg id's from the mailbox. That's used for clearning out the stuck ones below.
            if (hs_rscheck($stuckrs)) {
                while ($row = $stuckrs->FetchRow()) {
                    $this->stuck_emails[$row['xStuckEmail']] = $row['sEmailMessageId'].$row['sEmailFrom'];
                }

                //Create list of all ID's in the mailbox for use when deleting stuck emails
                $msgcount = $this->messageCount();
                for ($mid = 1; $mid <= $msgcount; $mid++) {
                    $info = @imap_headerinfo($this->conn, $mid);
                    $this->orig_msg_ids[] = $info->message_id.$info->from[0]->mailbox.'@'.$info->from[0]->host;
                }
            }

            return true;
        }
    }

    /**
     * Message count.
     *
     * @return    int
     */
    public function messageCount()
    {
        return @imap_num_msg($this->conn);
    }

    /**
     * Parse the message and store in mailbox message array.
     */
    public function initMessage($id, $timer = false)
    {
        //Get header info on message to see if it's stuck
        $msg_info = @imap_headerinfo($this->conn, $id);

        //Check if message is stuck (if we're seeing the same ID/from again then it is)
        $from = $msg_info->from[0]->mailbox.'@'.$msg_info->from[0]->host;
        $unique_id = $msg_info->message_id.$from;

        if (in_array($unique_id, $this->stuck_emails)) { //We already know this is a stuck email so skip it
            //We hit this one so skip it.
            return false;
        } elseif ($this->mailbox_info['sLastImportMessageId'] == $msg_info->message_id && $this->mailbox_info['sLastImportFrom'] == $from && $this->mailbox_info['iLastImportAttemptCt'] < 3) {
            //This is one that's been stuck already, but we haven't hit the CT yet that causes us to log it in stuck emails so just increment the count
            $GLOBALS['DB']->Execute('UPDATE HS_Mailboxes SET iLastImportAttemptCt = '.$this->mailbox_info['iLastImportAttemptCt'].' + 1 WHERE HS_Mailboxes.xMailbox = ?', [$this->mailbox_info['xMailbox']]);

            if ($this->debug) {
                echo '>>>> Possible stuck email. Attempting to use text version and skip Tidy if present'."\n";
            }
        } elseif ($this->mailbox_info['sLastImportMessageId'] == $msg_info->message_id && $this->mailbox_info['sLastImportFrom'] == $from && $this->mailbox_info['iLastImportAttemptCt'] >= 3) {
            //This is a new stuck email so log it.
            $GLOBALS['DB']->Execute('INSERT INTO HS_Mailboxes_StuckEmails (xMailbox,sEmailMessageId,sEmailFrom,sEmailDate) VALUES (?,?,?,?)', [$this->mailbox_info['xMailbox'], $msg_info->message_id, $from, $msg_info->date]);

            return false;
        } else {
            //log in mailbox info as this message being last imported. This way if it fails we can check here and see it's stuck
            $GLOBALS['DB']->Execute('UPDATE HS_Mailboxes SET sLastImportMessageId = ?, sLastImportFrom = ?, iLastImportAttemptCt = 1 WHERE HS_Mailboxes.xMailbox = ?', [$msg_info->message_id, $from, $this->mailbox_info['xMailbox']]);
        }

        //Use mailparse if available
        if (function_exists('mailparse_msg_parse_file') && $this->writable_upload_dir) {
            $parse = new \HS\Mail\Imap\Message\MailParse($this->debug, $timer);
        } else {
            $parse = new \HS\Mail\Imap\Message\Parse($this->debug, $timer);
        }

        return $parse->email($id, $this->conn);
    }

    /**
     * Expunge all messages marked for deletion on the server.
     *
     * @return    int
     */
    public function expunge()
    {
        imap_expunge($this->conn);
    }

    /**
     * Clear any email marked as stuck.
     *
     * @return    none
     */
    public function clear_stuck_emails()
    {
        //If we have some stuck emails ask the mail server if they're still there and if not delete them.
        //We can't do this any other way as there's a setting which users have to only allow x imports per tasks run. If that value is low a stuck email could "hide" beyond that value and
        //would then get hit again on the next run
        if (! empty($this->stuck_emails)) {
            foreach ($this->stuck_emails as $key=>$v) {
                //If stuck email ID not in original message id's for this mailbox then delete from stuck emails
                if (! in_array($v, $this->orig_msg_ids)) {
                    $GLOBALS['DB']->Execute('DELETE FROM HS_Mailboxes_StuckEmails WHERE HS_Mailboxes_StuckEmails.xStuckEmail = ?', [$key]);
                    \Facades\HS\Cache\Manager::forget(\HS\Cache\Manager::CACHE_STUCKEMAILS_KEY);
                }
            }
        }
    }

    /**
     * Close the IMAP connection.
     *
     * @return    bool
     */
    public function close()
    {
        //Log errors besides connection errors which are already handled elsewhere
        $er = [];
        $imap_errors = imap_errors();
        if (! empty($imap_errors)) {
            foreach ($imap_errors as $k=>$v) {
                if (strpos($v, 'Connection') === false) {
                    $er[] = $v.' (mailbox: '.$this->mailbox_info['xMailbox'].')';
                }
            }
            if (! empty($er)) {
                errorLog(implode(';', array_unique($er)), 'Email Importing', __FILE__, __LINE__);
            }
        }

        //Close connection
        imap_close($this->conn);
    }
}
